﻿using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;

namespace Wiedemann
{
    class SingleConnection
    {
        public MySqlConnection Connection = null;
        public String ConnectionErrorString = "";

        public SingleConnection(String dbServer,
            String dbUser, String dbPassword, String dbDatabase,
            String dbInitCommand, String dbConnectionStringSuffix)
        {
            if (IsConnectionOpen)
                Connection.Close();

            ConnectionErrorString = "";
            String connStr = String.Format("server={0}; user id={1}; password={2}; database={3}; pooling=false;",
                dbServer, dbUser, dbPassword, dbDatabase);
            connStr += dbConnectionStringSuffix;
            
            try
            {
                Connection = new MySqlConnection(connStr);
                Connection.Open();

                if (dbInitCommand.Length > 0)
                {
                    MySqlCommand cmdIniCP = new MySqlCommand(dbInitCommand, Connection);
                    cmdIniCP.ExecuteNonQuery();
                }
            }
            catch (MySqlException ex)
            {
                ConnectionErrorString = ex.Message;
                if (Connection != null)
                {
                    Connection.Close();
                    Connection = null;
                }
            }
        }

        public bool IsConnectionOpen
        {
            get
            {
                if (Connection == null)
                    return false;

                if (Connection.State == System.Data.ConnectionState.Open)
                    return true;
                else
                    return false;
            }
        }
    }

    public class ConnectionPool
    {
        static readonly object padlock = new object();
        static ConnectionPool instance = null;
        static Dictionary<String, SingleConnection> Connections = new Dictionary<String, SingleConnection>();

        // jedina instance tridy ConnectionPool spolecna celemu programu
        public static ConnectionPool Instance
        {
            get
            {
                lock (padlock)
                {
                    if (instance == null)
                    {
                        instance = new ConnectionPool();
                    }

                    return instance;
                }
            }
        }

        // konstruktor tridy volany pri 1. pouziti vlastnosti Instance
        private ConnectionPool()
        {
            // odstraneni vsech spojeni na databazi v poli
            Connections.Clear();
        }

        // escape uvozovek a stredniku
        static public String EscapeData(String srcStr)
        {
            String retStr = srcStr.Trim();
            retStr = retStr.Replace("\\", "\\\\");
            retStr = retStr.Replace("'", "\\'");
            retStr = retStr.Replace("\"", "\\\"");

            return retStr;
        }

        // vraci MySqlConnection urciteho spojeni
        static public MySqlConnection GetConnection(String ConnectionName)
        {
            MySqlConnection conn = null;
            try
            {
                conn = Connections[ConnectionName].Connection;
            }
            catch (KeyNotFoundException)
            {
                return null;
            }
            return conn;
        }

        // vytvori nove spojeni s mySQL v poli
        static public MySqlConnection AddConnection(String ConnectionName,
            String ConnectionServer, String ConnectionUser,
            String ConnectionPassword, String ConnectionDatabase,
            String ConnectionInitCommand, String ConnectionConnectionStringSuffix,
            out String ErrorString)
        {
            ErrorString = "";
            // test, jestli toto spojeni jiz existuje
            MySqlConnection conn = GetConnection(ConnectionName);
            if (conn == null)
            {
                // vytvoreni noveho
                SingleConnection singConn = new SingleConnection(ConnectionServer,
                    ConnectionUser, ConnectionPassword, ConnectionDatabase,
                    ConnectionInitCommand, ConnectionConnectionStringSuffix);
                // vyskytla se chyba?
                if (singConn.ConnectionErrorString.Length > 0)
                {
                    // ulozeni popisu chyby
                    ErrorString = singConn.ConnectionErrorString;
                    // funkce vraci null (nastaveno vysledkem volani funkce GetConnection(ConnectionName))
                }
                else
                {
                    // pridani noveho spojeni do pole
                    Connections.Add(ConnectionName, singConn);
                    conn = GetConnection(ConnectionName);
                }
            }

            return conn;
        }
    }

    public class ConnectionFailedException : System.Exception
    {
        public ConnectionFailedException()
        {
        }

        public ConnectionFailedException(string message)
            : base(message)
        {
        }
    }
}
